@@ -1,8 +1,6 @@
-
 #### A helper file containing math routines
 # Default python modules
 # Debugging
-from pdb import set_trace as stop
 import warnings
 import sys
 
@@ -27,13 +25,24 @@ import numba
 import numba.types as nt
 from llc import jit_filter_function
 
-def rms_loss_creator(func):
-    
-    def _rms_loss(pars, x, data):
-        model = func(x, *pars)
-        return rmsloss(x, data)
-    
-    return _rms_loss
+def compute_R2_stat(y1, y2, w=None):
+    if w is None:
+        w = np.ones_like(y1)
+    w = w / np.nansum(w)
+    y1bar = weighted_mean(y1, w)
+    sstot = np.nansum((y1 - ybar)**2)
+    ssres = np.nansum((y1 - y2)**2)
+    return 1 - (ss_res / ss_tot)
+
+
+def outer_fun(fun, x, y):
+    n1 = len(x)
+    n2 = len(x)
+    out = np.zeros((n1, n2))
+    for i in range(n1):
+        for j in range(n2):
+            out[i, j] = fun(x[i], y[j])
+    return out
 
 def rmsloss(x, y, weights=None, flag_worst=0, remove_edges=0):
     
@@ -138,7 +147,7 @@ def rolling_clip(x, y, weights=None, width=None, method='median', nsigma=3, perc
                 
     return mask
 
-def doppler_shift(wave, vel, wave_out=None, flux=None, interp='linear', kind='exp'):
+def doppler_shift(wave, vel, wave_out=None, flux=None, interp='cspline', kind='exp'):
     
     if wave_out is None:
         wave_out = wave
@@ -151,8 +160,8 @@ def doppler_shift(wave, vel, wave_out=None, flux=None, interp='linear', kind='ex
     if interp is None and flux is None:
         return wave_shifted
     good = np.where(np.isfinite(wave_shifted) & np.isfinite(flux))[0]
-    if interp == 'cubic':
-        flux_out = scipy.interpolate.CubicSpline(wave_shifted[good], flux[good], extrapolate=False)(wave_out)
+    if interp == 'cspline':
+        flux_out = cspline_interp(wave_shifted, flux, wave_out)
     elif interp == 'akima':
         flux_out = scipy.interpolate.Akima1DInterpolator(wave_shifted[good], flux[good])(wave_out)
     elif interp == 'pchip':
@@ -162,6 +171,21 @@ def doppler_shift(wave, vel, wave_out=None, flux=None, interp='linear', kind='ex
     return flux_out
     
 
+def lin_interp(x, y, xnew):
+    return np.interp(xnew, x, y, left=np.nan, right=np.nan)
+
+def cspline_interp(x, y, xnew):
+    good = np.where(np.isfinite(x) & np.isfinite(y))[0]
+    return scipy.interpolate.CubicSpline(x[good], y[good], extrapolate=False)(xnew)
+
+def cspline_fit(x, y, knots, weights=None):
+    if weights is None:
+        weights = np.ones_like(y)
+    good = np.where(np.isfinite(x) & np.isfinite(y) & np.isfinite(weights) & (weights > 0))[0]
+    xx, yy, ww = x[good], y[good], weights[good]
+    _cspline_fit = scipy.interpolate.LSQUnivariateSpline(xx, yy, t=knots[1:-1], w=ww, k=3, ext=1)
+    return _cspline_fit
+
 @njit
 def _dop_shift_SR(wave, vel):
     z = vel / cs.c
@@ -315,6 +339,53 @@ def median_filter2d(x, width, preserve_nans=True):
         
     return out
 
+@njit
+def convolve1d(x, k, padleft, padright):
+
+    # Length of each
+    nx = len(x)
+    nk = len(k)
+
+    # The left and right padding
+    n_pad = numba.int64(nk / 2)
+
+    # Output vector
+    out = np.zeros(x.shape)
+
+    # Flip the kernel (just a view, no new vector is allocated)
+    kf = k[::-1]
+
+    # Left values
+    for i in range(n_pad):
+        s = 0.0
+        for j in range(nk):
+            ii = i - n_pad + j
+            if ii < 0:
+                s += padleft * kf[j]
+            else:
+                s += x[ii] * kf[j]
+        out[i] = s
+
+    # Middle values
+    for i in range(n_pad, nx-n_pad):
+        s = 0.0
+        for j in range(nk):
+            s += x[i - n_pad + j] * kf[j]
+        out[i] = s
+
+    # Right values
+    for i in range(nx-n_pad, nx):
+        s = 0.0
+        for j in range(nk):
+            ii = i - n_pad + j
+            if ii > nx - 1:
+                s += padleft * kf[j]
+            else:
+                s += x[ii] * kf[j]
+        out[i] = s
+
+    # Return out
+    return out
 
 def convolve_flux(wave, flux, R=None, width=None, interp=None, lsf=None, croplsf=False):
     """Convolves flux.
@@ -410,6 +481,7 @@ def convolve_flux(wave, flux, R=None, width=None, interp=None, lsf=None, croplsf
 
     # Convolve
     fluxlinc = np.convolve(fluxlinp, lsf, mode='valid')
+    #fluxlinc = convolve1d(fluxlin, lsf, fluxlin[0], fluxlin[-1])
     
     # Interpolate back to the default grid
     if interp:
@@ -560,7 +632,7 @@ def weighted_stddev_mumod(x, w, mu):
     return np.sqrt(var)
 
 # This calculates the weighted mean of array x with weights w
-@jit
+#@jit
 def weighted_mean(x, w):
     """Computes the weighted mean of a dataset.
 
@@ -573,6 +645,56 @@ def weighted_mean(x, w):
     """
     return np.nansum(x * w) / np.nansum(w)
 
+def weighted_combine(y, w, yerr=None, err_type="Poisson"):
+    """Performs a weighted coadd.
+
+    Args:
+        y (np.ndarray): The data to coadd.
+        w (np.ndarray): The weights.
+        yerr (np.ndarray): The correspinding error bars. Defaults to None.
+        err_type (str): The method to determine error bars. If "empirical", error bars are computed by considering the weighted stddev of the appropriate data points. If "poisson", error bars are computed by taking the mean of the appropriate data errors divided by the sqrt(N), where N is the number of data points. If there are only two data points, the poisson method is used regardless.
+    """
+    
+    # Determine how many good data points.
+    good = np.where(w > 0)[0]
+    n_good = len(good)
+
+    # If none are good, return nan
+    if n_good == 0:
+        yc, yc_unc = np.nan, np.nan
+        
+    # If only one is good, the mean is the only good value, and the error is the only good error.
+    # If no error is provided, nan is returned.
+    elif n_good == 1:
+        yc = y[good[0]]
+        if yerr is not None:
+            yc_unc = yerr[good[0]]
+        else:
+            yc_unc = np.nan
+            
+    # If only two are good, the mean is the weighted mean.
+    # Error is computed from existing errors if provided, or empirically otherwise.
+    elif n_good == 2:
+        yc = weighted_mean(y[good].flatten(), w[good].flatten())
+        if yerr is not None:
+            yc_unc = np.nanmean(yerr[good].flatten()) /  np.sqrt(2)
+        else:
+            yc_unc = weighted_stddev(y[good].flatten(), w[good].flatten()) / np.sqrt(2)
+    
+    # With >= 3 useful points, the mean is the weighted mean.
+    # Error is computed from err_type
+    else:
+        yc = weighted_mean(y[good].flatten(), w[good].flatten())
+        if err_type.lower() == "poisson":
+            if yerr is not None:
+                yc_unc = np.nanmean(yerr[good].flatten()) / np.sqrt(n_good)
+            else:
+                yc_unc = weighted_stddev(y[good].flatten(), w[good].flatten()) / np.sqrt(n_good)
+        else:
+            yc_unc = weighted_stddev(y[good].flatten(), w[good].flatten()) / np.sqrt(n_good)
+        
+    return yc, yc_unc
+
 # Rolling function f over a window given w of y given the independent variable x
 def rolling_fun_true_window(f, x, y, w):
     """Computes a filter over the data using windows determined from a proper independent variable.
