@@ -1,3 +1,5 @@
+
+
 class Trace:
     
     def __init__(self):
@@ -23,23 +25,15 @@ class SkyTrace:
     def __repr__(self):
         return "Stellar Trace"
     
-class EmissionTrace:
+class SparseTrace:
     
     def __init__(self):
         pass
     
     def __repr__(self):
-        return "Sparse Emission Line Trace"
-    
-class CombTrace:
-    
-    def __init__(self):
-        pass
-    
-    def __repr__(self):
-        return "Laser Frequency Comb Trace"
-    
+        return "Sparse Line Trace"
     
+
 class SlitFlatTrace:
     
     def __init__(self):
