@@ -1,22 +1,14 @@
 # Python built in modules
 import copy
-import glob # File searching
 import os # Making directories
-import importlib.util # importing other modules from files
-import warnings # ignore warnings
 import time # Time the code
 import pickle
-import inspect
-import multiprocessing # parallelization on a single node
-import sys # sys utils
-from sys import platform # plotting backend
 
 # Graphics
 import matplotlib # to set the backend
 import matplotlib.pyplot as plt # Plotting
 import pychell
 plt.style.use(os.path.dirname(pychell.__file__) + os.sep + "gadfly_stylesheet.mplstyle")
-from matplotlib import cm
 
 # Multiprocessing
 from joblib import Parallel, delayed
@@ -25,13 +17,13 @@ import tqdm
 # Science/math
 from scipy import constants as cs # cs.c = speed of light in m/s
 import numpy as np # Math, Arrays
-import scipy.interpolate # Cubic interpolation, Akima interpolation
 
 # llvm
 from numba import njit, jit, prange
 
 # User defined
 import pychell.maths as pcmath
+from pychell.maths import cspline_interp, lin_interp
 import pychell.rvs.template_augmenter as pcaugmenter
 import pychell.rvs.model_components as pcmodels
 import pychell.data as pcdata
@@ -95,7 +87,7 @@ class ForwardModels(list):
         # Load templates
         self.load_templates()
             
-        print_init_summary(self)
+        self.print_init_summary()
         
         # Initial optimization to guess some parameters
         self.init_optimize()
@@ -137,6 +129,18 @@ class ForwardModels(list):
             for model in fwm.models_dict:
                 fwm.models_dict[model].update(fwm, iter_index)
     
+    def print_init_summary(self):
+        """Print a summary for this run.
+        """
+        # Print summary
+        print('***************************************', flush=True)
+        print('** Target: ' + self.star_name, flush=True)
+        print('** Spectrograph: ' + self.observatory['name'] + ' / ' + self.spectrograph, flush=True)
+        print('** Observations: ' + str(self.n_spec) + ' spectra, ' + str(self.n_nights) + ' nights', flush=True)
+        print('** Echelle Order: ' + str(self.order_num), flush=True)
+        print('** TAG: ' + self.tag, flush=True)
+        print('** N Iterations: ' + str(self.n_template_fits), flush=True)
+        print('***************************************', flush=True)
     
     def init_rvs(self):
         
@@ -199,7 +203,7 @@ class ForwardModels(list):
     
     def init_optimize(self):
         if self[0].models_dict['star'].from_synthetic:
-            self.get_init_stellar_vel_ccf()
+            self.get_init_star_vel_ccf()
         for fwm in self:
             fwm.init_optimize(self.templates_dict)
 
@@ -267,7 +271,8 @@ class ForwardModels(list):
                 iter_pass.append((self[ispec], self.templates_dict, iter_index, self.rvs_dict['xcorr_options']))
 
             # Cross Correlate in Parallel
-            ccf_results = Parallel(n_jobs=self.n_cores, verbose=0, batch_size=1)(delayed(self[0].cross_correlate_all_regions_wrapper)(*iter_pass[ispec]) for ispec in tqdm.tqdm(range(self.n_spec)))
+            #ccf_results = Parallel(n_jobs=self.n_cores, verbose=0, batch_size=1)(delayed(self[0].cross_correlate_all_regions_wrapper)(*iter_pass[ispec]) for ispec in tqdm.tqdm(range(self.n_spec)))
+            ccf_results = Parallel(n_jobs=self.n_cores, verbose=0, batch_size=1)(delayed(self[0].cross_correlate_all_regions_wrapper)(*iter_pass[ispec]) for ispec in range(self.n_spec))
             
         else:
             ccf_results = [self[0].cross_correlate_all_regions_wrapper(self[ispec], self.templates_dict, iter_index, self.rvs_dict['xcorr_options']) for ispec in range(self.n_spec)]
@@ -338,6 +343,7 @@ class ForwardModels(list):
         # Alias
         rvs_dict = self.rvs_dict
         
+        # Plot the individual rvs for each chunk (n_chunks * n_spec)
         for ichunk in range(self.n_chunks):
         
             # Individual rvs from nelder mead fitting
@@ -399,13 +405,12 @@ class ForwardModels(list):
                 plt.title(self[0].star_name + ' CCF Bisector Spans Order ' + str(self.order_num) + ', Iteration ' + str(iter_index + 1), fontweight='bold')
                 plt.xlabel('X Corr RV [m/s]', fontweight='bold')
                 plt.ylabel('Bisector Span [m/s]', fontweight='bold')
-                plt.tight_layout()
+            plt.tight_layout()
             fname = self.run_output_path + self.o_folder + 'RVs' + os.sep + self.tag + '_bisectorspans_ord' + str(self.order_num) + '_iter' + str(iter_index + 1) + '.png'
             plt.savefig(fname)
             plt.close()
-    
         
-    def get_init_stellar_vel_ccf(self):
+    def get_init_star_vel_ccf(self):
         """Cross correlation wrapper for all spectra.
 
         Args:
@@ -427,15 +432,21 @@ class ForwardModels(list):
                 iter_pass.append((self[ispec], self.templates_dict, self[ispec].sregion_order))
 
             # Cross Correlate in Parallel
-            ccf_results = Parallel(n_jobs=self.n_cores, verbose=0, batch_size=1)(delayed(ccf_method)(*iter_pass[ispec]) for ispec in tqdm.tqdm(range(self.n_spec)))
+            #ccf_results = Parallel(n_jobs=self.n_cores, verbose=0, batch_size=1)(delayed(ccf_method)(*iter_pass[ispec]) for ispec in tqdm.tqdm(range(self.n_spec)))
+            ccf_results = Parallel(n_jobs=self.n_cores, verbose=0, batch_size=1)(delayed(ccf_method)(*iter_pass[ispec]) for ispec in range(self.n_spec))
         else:
-            ccf_results = [ccf_method(fwm, self.templates_dict, fwm.sregion_order) for fwm in tqdm.tqdm(self)]
+            #ccf_results = [ccf_method(fwm, self.templates_dict, fwm.sregion_order) for fwm in tqdm.tqdm(self)]
+            ccf_results = [ccf_method(fwm, self.templates_dict, fwm.sregion_order) for fwm in self]
             
         for ispec, fwm in enumerate(self):
-            fwm.initial_parameters[fwm.models_dict['star'].par_names[0]].value = ccf_results[ispec]
+            if ccf_results[ispec] != 0:
+                fwm.initial_parameters[fwm.models_dict['star'].par_names[0]].value = ccf_results[ispec]
+            else:
+                fwm.initial_parameters[fwm.models_dict['star'].par_names[0]].value = ccf_results[ispec] + 10
                 
         print('Cross Correlation Finished in ' + str(round((stopwatch.time_since())/60, 3)) + ' min ', flush=True)
 
+
 class ForwardModel:
     
     def __init__(self, input_file, config, model_blueprints, parser, order_num, spec_num):
@@ -476,7 +487,7 @@ class ForwardModel:
         # Storage arrays after each iteration
         # Each entry is a tuple for each iteration: (best_fit_pars, RMS, FCALLS)
         self.opt_results = []
-    
+
     def init_chunks(self, model_blueprints):
         good = np.where(self.data.mask)[0]
         order_pixmin, order_pixmax = good[0], good[-1]
@@ -490,7 +501,7 @@ class ForwardModel:
             pixmin, pixmax = stitch_points_pix[ichunk], stitch_points_pix[ichunk + 1]
             wavemin, wavemax = wave_estimate[pixmin], wave_estimate[pixmax]
             self.chunk_regions.append(pcutils.SpectralRegion(pixmin, pixmax, wavemin, wavemax, label=ichunk))
-        
+
     def init_models(self, config, model_blueprints):
         
         # A dictionary to store model components
@@ -520,18 +531,17 @@ class ForwardModel:
             model_class = getattr(pcmodels, model_blueprints[blueprint]['class'])
             self.models_dict[blueprint] = model_class(self, model_blueprints[blueprint])
 
-
     def init_parameters(self, sregion=None):
         self.initial_parameters = OptimParameters.Parameters()
         for model in self.models_dict:
             self.models_dict[model].init_parameters(self)
         self.initial_parameters.sanity_lock()
- 
+
     def init_optimize(self, templates_dict):
         templates_dict_chunked = self.init_chunk(templates_dict, self.sregion_order)
         for model in self.models_dict:
             self.models_dict[model].init_optimize(self, templates_dict_chunked)
-                
+
     # Prints the models and corresponding parameters after each fit if verbose_print=True
     def pretty_print(self):
         # Loop over models
@@ -547,7 +557,7 @@ class ForwardModel:
                     print(self.initial_parameters[pname], flush=True)
                 else:
                     print(self.opt_results[-1][-1]['xbest'][pname], flush=True)
-                
+
     def set_parameters(self, pars):
         self.initial_parameters.update(pars)
     
@@ -566,7 +576,6 @@ class ForwardModel:
         fig, axarr = plt.subplots(self.n_chunks, 1, figsize=(int(plot_width / dpi), int(self.n_chunks * plot_height / dpi)), dpi=dpi, constrained_layout=True)
         axarr = np.atleast_1d(axarr)
         
-            
         for ichunk, sregion in enumerate(self.chunk_regions):
             
             # The best fit parameters
@@ -619,28 +628,28 @@ class ForwardModel:
                 if self.models_dict['star'].enabled:
                     star_flux_hr = self.models_dict['star'].build(pars, templates_dict_chunked['star'], self.model_wave)
                     star_convolved = self.models_dict['lsf'].convolve_flux(star_flux_hr, lsf=lsf)
-                    star_flux_lr = np.interp(wave_data, self.model_wave, star_convolved, left=np.nan, right=np.nan)
+                    star_flux_lr = cspline_interp(self.model_wave, star_convolved, wave_data)
                     axarr[ichunk].plot(wave_data / 10, star_flux_lr - 1.1, label='Star', lw=0.8, color='deeppink', alpha=0.8)
                 
                 # Tellurics
                 if 'tellurics' in self.models_dict and self.models_dict['tellurics'].enabled:
                     tellurics = self.models_dict['tellurics'].build(pars, templates_dict_chunked['tellurics'], self.model_wave)
                     tellurics_convolved = self.models_dict['lsf'].convolve_flux(tellurics, lsf=lsf)
-                    tell_flux_lr = np.interp(wave_data, self.model_wave, tellurics_convolved, left=np.nan, right=np.nan)
+                    tell_flux_lr = cspline_interp(self.model_wave, tellurics_convolved, wave_data)
                     axarr[ichunk].plot(wave_data / 10, tell_flux_lr - 1.1, label='Tellurics', lw=0.8, color='indigo', alpha=0.8)
                 
                 # Gas Cell
                 if 'gas_cell' in self.models_dict and self.models_dict['gas_cell'].enabled:
                     gas_flux_hr = self.models_dict['gas_cell'].build(pars, templates_dict_chunked['gas_cell'], self.model_wave)
                     gas_cell_convolved = self.models_dict['lsf'].convolve_flux(gas_flux_hr, lsf=lsf)
-                    gas_flux_lr = np.interp(wave_data, self.model_wave, gas_cell_convolved, left=np.nan, right=np.nan)
+                    gas_flux_lr = cspline_interp(self.model_wave, gas_cell_convolved, wave_data)
                     axarr[ichunk].plot(wave_data / 10, gas_flux_lr - 1.1, label='Gas Cell', lw=0.8, color='green', alpha=0.8)
                 axarr[ichunk].set_ylim(-1.1, 1.1)
                 
                 # Residual lab flux
                 if 'residual_lab' in templates_dict:
                     res_hr = templates_dict['residual_lab'][:, 1]
-                    res_lr = np.interp(wave_data, self.model_wave, res_hr, left=np.nan, right=np.nan)
+                    res_lr = cspline_interp(self.model_wave, res_hr, wave_data)
                     ax.plot(wave_data / 10, res_lr - 0.1, label='Lab Frame Coherence', lw=0.8, color='darkred', alpha=0.8)
                     
                 axarr[ichunk].legend(prop={'size': 8}, loc='lower right')
@@ -654,10 +663,9 @@ class ForwardModel:
         
         # Save
         #plt.subplots_adjust(left=0.05, bottom=0.05, right=None, top=0.95, wspace=None, hspace=None)
-        plt.tight_layout()
         plt.savefig(fname)
         plt.close()
-        
+
     def init_chunk(self, templates_dict, sregion=None):
         
         if sregion is None:
@@ -690,11 +698,11 @@ class ForwardModel:
         fname = self.run_output_path + self.o_folder + 'ForwardModels' + os.sep + self.tag + '_forward_model_ord' + str(self.order_num) + '_spec' + str(self.spec_num) + '.pkl'
         with open(fname, 'wb') as f:
             pickle.dump(self, f)
-            
+
     # Gets the night which corresponds to the spec index
     def get_thisnight_index(self, n_obs_nights):
         return self.get_night_index(self.spec_index, n_obs_nights)
-    
+
     # Gets the night which corresponds to the spec index
     @staticmethod
     def get_night_index(spec_index, n_obs_nights):
@@ -705,8 +713,8 @@ class ForwardModel:
             if spec_index < running_spec_index:
                 return inight
             running_spec_index += n_obs_nights[inight+1]
-    
-    
+
+
     # Gets the indices of spectra for a certain night. (zero based)
     def get_all_spec_indices_from_thisnight(self, n_obs_nights):
         night_index = self.get_thisnight_index(n_obs_nights)
@@ -905,7 +913,7 @@ class ForwardModel:
         wavelength_solution = self.models_dict['wavelength_solution'].build(pars)
 
         # Interpolate high res model onto data grid
-        model_lr = np.interp(wavelength_solution, self.model_wave, model, left=np.nan, right=np.nan)
+        model_lr = cspline_interp(self.model_wave, model, wavelength_solution)
         
         if self.debug:
             breakpoint()
@@ -942,51 +950,34 @@ class ForwardModel:
         wavelength_solution = self.models_dict['wavelength_solution'].build(pars)
 
         # Interpolate high res model onto data grid
-        model_lr = np.interp(wavelength_solution, self.model_wave, model, left=np.nan, right=np.nan)
+        model_lr = cspline_interp(self.model_wave, model, wavelength_solution)
         
         if self.debug:
             breakpoint()
 
         return wavelength_solution, model_lr
   
-  
 class iSHELLForwardModel(ForwardModel):
     pass
-    
-    
+        
 class CHIRONForwardModel(ForwardModel):
     pass
 
-
 class PARVIForwardModel(ForwardModel):
     pass
 
+class IRDForwardModel(ForwardModel):
+    pass
 
 class MinervaAustralisForwardModel(ForwardModel):
     pass
     
-    
 class MinervaNorthForwardModel(ForwardModel):
     pass
     
-    
 class NIRSPECForwardModel(ForwardModel):
     pass
     
 class SimulatedForwardModel(ForwardModel):
     pass
-    
-    
-# Post init summary
-def print_init_summary(forward_model):
-    """Print a summary for this run.
-    """
-    # Print summary
-    print('***************************************', flush=True)
-    print('** Target: ' + forward_model.star_name, flush=True)
-    print('** Spectrograph: ' + forward_model.observatory['name'] + ' / ' + forward_model.spectrograph, flush=True)
-    print('** Observations: ' + str(forward_model.n_spec) + ' spectra, ' + str(forward_model.n_nights) + ' nights', flush=True)
-    print('** Echelle Order: ' + str(forward_model.order_num), flush=True)
-    print('** TAG: ' + forward_model.tag, flush=True)
-    print('** N Iterations: ' + str(forward_model.n_template_fits), flush=True)
-    print('***************************************', flush=True)
\ No newline at end of file
+    
\ No newline at end of file
