@@ -1,34 +1,22 @@
-# Python built in modules
-from collections import OrderedDict
-from abc import ABC, abstractmethod  # Abstract classes
-import inspect
-import glob
-
 # Science/math
 from scipy import constants as cs  # cs.c = speed of light in m/s
 import numpy as np  # Math, Arrays
-import sys
 from scipy.special import legendre
-import scipy.interpolate  # Cubic, Akima, Pchip interpolation
 
 # Graphics
 import matplotlib.pyplot as plt
 
-# llvm
-from numba import njit, jit, jitclass
-import numba
-
-# User defined
+# pychell
 import pychell.maths as pcmath
+from pychell.maths import cspline_interp
 import pychell.rvs.template_augmenter as pcaugmenter
 import optimparameters.parameters as OptimParameters
 
 # NOTE: The idea is to first define crude "quasi abstract" classes
-# Then define a quasi abstract class for each sub type.
-# Then define a concrete class for particular models.
+# Then define a quasi abstract class for each sub type (continuum, star, gas cell, tellurics, etc).
+# Then define a concrete class for particular models (SplineContinuum, TAPASTellurics, etc).
 
 #### Quasi Abstract Classes ####
-
 class SpectralComponent:
     """Base class for a general spectral component model.
 
@@ -191,11 +179,10 @@ class TemplateMult(MultModelComponent):
     def normalize_template(self, forward_model, wave, flux, uniform=False):
         
         if not uniform:
-            good = np.where(np.isfinite(wave) & np.isfinite(flux))[0]
             dl = np.nanmedian(np.diff(wave))
             wave_min, wave_max = np.nanmin(wave), np.nanmax(wave)
             wave_lin = np.arange(wave_min, wave_max, dl)
-            flux_lin = scipy.interpolate.CubicSpline(wave[good], flux[good], extrapolate=False)(wave_lin)
+            flux_lin = cspline_interp(wave, flux, wave_lin)
         else:
             flux_lin = flux
         
@@ -251,8 +238,7 @@ class ContinuumModel(EmpiricalMult):
 
         good = np.where(np.isfinite(continuum_coarse))[0]
         inds = np.linspace(good[0], good[-1], num=n_knots).astype(int)
-        cspline = scipy.interpolate.CubicSpline(wave[inds], continuum_coarse[inds], extrapolate=False, bc_type='not-a-knot')
-        continuum = cspline(wave)
+        continuum = cspline_interp(wave[inds], continuum_coarse[inds], wave)
         return continuum
 
     @staticmethod
@@ -410,7 +396,7 @@ class SplineContinuum(ContinuumModel):
         spline_pars = np.array([pars[self.par_names[i]].value for i in range(self.n_spline_pars)], dtype=np.float64)
 
         # Build
-        spline_cont = scipy.interpolate.CubicSpline(self.spline_wave_set_points, spline_pars, extrapolate=False, bc_type='not-a-knot')(wave_final)
+        spline_cont = cspline_interp(self.spline_wave_set_points, spline_pars, wave_final)
         
         return spline_cont
 
@@ -459,7 +445,7 @@ class DynamicGasCell(GasCell):
         wave, flux = template[:, 0], template[:, 1]
         wave = wave + pars[self.par_names[0]].value
         flux = flux ** pars[self.par_names[1]].value
-        return np.interp(wave_final, wave, flux, left=flux[0], right=flux[-1])
+        return cspline_interp(wave, flux, wave_final)
 
     def init_parameters(self, forward_model):
         forward_model.initial_parameters.add_parameter(OptimParameters.Parameter(name=self.par_names[0], value=self.blueprint['shift'][1], minv=self.blueprint['shift'][0], maxv=self.blueprint['shift'][2], vary=True))
@@ -483,7 +469,7 @@ class CHIRONGasCell(DynamicGasCell):
         wave, flux = template[:, 0], template[:, 1]
         wave = wave + pars[self.par_names[0]].value
         flux = flux ** pars[self.par_names[1]].value
-        return np.interp(wave_final, wave, flux, left=flux[0], right=flux[-1])
+        return cspline_interp(wave, flux, wave_final)
 
     def init_parameters(self, forward_model):
         shift = self.blueprint['shifts'][self.order_num - 1]
@@ -508,7 +494,7 @@ class PerfectGasCell(GasCell):
 
     def build(self, pars, template, wave_final):
         wave, flux = template[:, 0], template[:, 1]
-        return np.interp(wave_final, wave, flux, left=flux[0], right=flux[-1])
+        return cspline_interp(wave, flux, wave_final)
 
 #### Star ####
 class Star(TemplateMult):
@@ -573,7 +559,7 @@ class AugmentedStar(Star):
 
     def build(self, pars, template, wave_final):
         wave, flux = template[:, 0], template[:, 1]
-        flux_shifted_interp = pcmath.doppler_shift(wave, pars[self.par_names[0]].value, wave_out=wave_final, flux=flux, interp='cubic')
+        flux_shifted_interp = pcmath.doppler_shift(wave, pars[self.par_names[0]].value, wave_out=wave_final, flux=flux, interp='cspline')
         return flux_shifted_interp
 
     def init_parameters(self, forward_model):
@@ -586,8 +572,7 @@ class AugmentedStar(Star):
             print('Loading in Synthetic Stellar Template', flush=True)
             template_raw = np.loadtxt(self.input_file, delimiter=',')
             wave, flux = template_raw[:, 0], template_raw[:, 1]
-            good = np.where(np.isfinite(wave) & np.isfinite(flux))[0]
-            flux_interp = scipy.interpolate.CubicSpline(wave[good], flux[good], extrapolate=False, bc_type='not-a-knot')(wave_uniform)
+            flux_interp = cspline_interp(wave, flux, wave_uniform)
             flux_interp /= pcmath.weighted_median(flux_interp, percentile=0.999)
             template = np.array([wave_uniform, flux_interp]).T
         else:
@@ -634,9 +619,9 @@ class TelluricsTAPAS(Tellurics):
         if self.airmass_enabled:
             flux *= self.build_component(pars, templates, 'airmass')
         if vel != 0:
-            flux = pcmath.doppler_shift(templates[:, 0], wave_out=wave_final, vel=vel, flux=flux, interp='linear')
+            flux = pcmath.doppler_shift(templates[:, 0], wave_out=wave_final, vel=vel, flux=flux, interp='cspline')
         else:
-            flux = np.interp(wave_final, templates[:, 0], flux, left=np.nan, right=np.nan)
+            flux = cspline_interp(templates[:, 0], flux, wave_final)
         return flux
 
     def build_component(self, pars, templates, component):
@@ -682,8 +667,7 @@ class TelluricsTAPAS(Tellurics):
             wave, _flux = tell['wave'], tell['flux']
             good = np.where((wave > forward_model.sregion_order.wavemin - pad) & (wave < forward_model.sregion_order.wavemax + pad))[0]
             wave, _flux = wave[good], _flux[good]
-            good = np.where(np.isfinite(wave) & np.isfinite(_flux))[0]
-            flux_airmass *= scipy.interpolate.CubicSpline(wave[good], _flux[good], extrapolate=False)(wave_water)
+            flux_airmass *= cspline_interp(wave, _flux, wave_water)
             
         templates[:, 2] = flux_airmass
             
@@ -702,16 +686,20 @@ class TelluricsTAPAS(Tellurics):
         yrange_airmass = self.template_yrange(forward_model, templates_dict["tellurics"][:, 0], templates_dict["tellurics"][:, 2],  sregion)
         if yrange_water > self.flag_thresh[0]:
             self.has_water_features = True
-            self.enable(forward_model, "water")
+            if self.enabled:
+                self.enable(forward_model, "water")
         else:
             self.has_water_features = False
-            self.disable(forward_model, "water")
+            if self.enabled:
+                self.disable(forward_model, "water")
         if yrange_airmass > self.flag_thresh[0]:
             self.has_airmass_features = True
-            self.enable(forward_model, "airmass")
+            if self.enabled:
+                self.enable(forward_model, "airmass")
         else:
             self.has_airmass_features = False
-            self.disable(forward_model, "airmass")
+            if self.enabled:
+                self.disable(forward_model, "airmass")
             
         if self.flag_and_ignore:
             self.flag_tellurics(forward_model, templates_dict)
@@ -724,13 +712,13 @@ class TelluricsTAPAS(Tellurics):
         yrange_water = self.template_yrange(forward_model, templates_dict["tellurics"][:, 0], templates_dict["tellurics"][:, 1], forward_model.sregion_order)
         yrange_airmass = self.template_yrange(forward_model, templates_dict["tellurics"][:, 0], templates_dict["tellurics"][:, 2], forward_model.sregion_order)
         
-        if yrange_water > self.flag_thresh[0]:
+        if yrange_water > self.flag_thresh[0] and self.enabled:
             self.has_water_features = True
             self.enable(forward_model, "water")
         else:
             self.has_water_features = False
             self.disable(forward_model, "water")
-        if yrange_airmass > self.flag_thresh[0]:
+        if yrange_airmass > self.flag_thresh[0] and self.enabled:
             self.has_airmass_features = True
             self.enable(forward_model, "airmass")
         else:
@@ -754,6 +742,7 @@ class TelluricsTAPAS(Tellurics):
         if self.water_enabled or self.airmass_enabled:
             self.enabled = True
             forward_model.initial_parameters[self.par_names[0]].vary = True
+            
         
     def disable(self, forward_model, component):
         if component == "water":
@@ -769,9 +758,9 @@ class TelluricsTAPAS(Tellurics):
     def flag_tellurics(self, forward_model, templates_dict):
         tell_flux_hr = self.build(forward_model.initial_parameters, templates_dict["tellurics"], forward_model.model_wave)
         wave_data = forward_model.models_dict["wavelength_solution"].build(forward_model.initial_parameters)
-        tell_flux_lr = np.interp(wave_data, forward_model.model_wave, tell_flux_hr, left=np.nan, right=np.nan)
+        tell_flux_lr = cspline_interp(forward_model.model_wave, tell_flux_hr, wave_final)
         tell_flux_lr / pcmath.weighted_median(tell_flux_lr, percentile=0.999)
-        bad = np.where(tell_flux_lr < 1 - self.thresh)[0]
+        bad = np.where(tell_flux_lr < self.thresh[1])[0]
         if bad.size > 0:
             forward_model.data.flux[forward_model.sregion_order.data_inds[bad]] = np.nan
             forward_model.data.flux_unc[forward_model.sregion_order.data_inds[bad]] = np.nan
@@ -1111,7 +1100,7 @@ class SplineWavelengthSolution(WavelengthSolution):
         spline_pars = np.array([pars[self.par_names[i]].value for i in range(self.n_spline_pars)], dtype=np.float64)
         
         # Build the spline model
-        spline_wave = scipy.interpolate.CubicSpline(self.spline_pixel_set_points, spline_pars + self.spline_wave_set_points, extrapolate=False, bc_type='not-a-knot')(pixel_grid)
+        spline_wave = cspline_interp(self.spline_pixel_set_points, spline_pars + self.spline_wave_set_points, pixel_grid)
         
         return spline_wave
 
@@ -1176,7 +1165,7 @@ class HybridWavelengthSolution(WavelengthSolution):
         else:
             pixel_grid = np.arange(self.sregion.pixmin, self.sregion.pixmax + 1)
             splines = np.array([pars[self.par_names[i]].value for i in range(self.n_splines + 1)], dtype=np.float64)
-            wave_spline = scipy.interpolate.CubicSpline(self.spline_pixel_set_points, splines, bc_type='not-a-knot', extrapolate=False)(pixel_grid)
+            wave_spline = cspline_interp(self.spline_pixel_set_points, splines, pixel_grid)
             return self.default_wave_grid[self.sregion.data_inds] + wave_spline
 
     def build_fake(self):
