@@ -1,5 +1,5 @@
+# Default python modules
 import os
-from pdb import set_trace as stop
 
 # Multiprocessing
 from joblib import Parallel, delayed
@@ -11,7 +11,7 @@ import scipy.stats
 import scipy.constants as cs
 
 # LLVM
-from numba import jit, njit, prange
+from numba import jit
 
 # Graphics
 import matplotlib.pyplot as plt
@@ -19,22 +19,20 @@ import pychell
 plt.style.use(os.path.dirname(pychell.__file__) + os.sep + "gadfly_stylesheet.mplstyle")
 from robustneldermead.neldermead import NelderMead
 import pychell.maths as pcmath
-import pychell.utils as pcutils
 import copy
 
 # Optimization
 from robustneldermead.neldermead import NelderMead
 
-
 def get_nightly_jds(jds, sep=0.5):
-    """Computes nightly (average) JDs (or BJDs) for a time-series observation over several nights.
+    """Computes nightly (average) JDs (or BJDs) for a time-series observation over many nights. Average times are computed from the mean of the considered times.
 
     Args:
         jds (np.ndarray): An array of sorted JDs (or BJDs).
-        sep (float): The minimum separation in days between two different nights of data, defaults to 0.5.
+        sep (float): The minimum separation in days between two different nights of data, defaults to 0.5 (half a day).
     Returns:
-        (np.ndarray): The average nightly jds.
-        (np.ndarray): The number of observations each night.
+        np.ndarray: The average nightly jds.
+        np.ndarray: The number of observations each night with data, of length n_nights.
     """
     
     # Number of spectra
@@ -64,8 +62,27 @@ def get_nightly_jds(jds, sep=0.5):
     n_obs_nights = np.array(n_obs_nights).astype(int)
 
     return jds_nightly, n_obs_nights
+        
+def nightly_iteration(n_obs_nights):
+    """A generator for iterating over observations within a given night.
 
+    Args:
+        n_obs_nights (np.ndarray): The number of observations on each night.
 
+    Yields:
+        int: The night index.
+        int: The index of the first observation for this night.
+        int: The index of the last observation for this night + 1. The additional + 1 is so one can index the array via array[f:l].
+    """
+    n_nights = len(n_obs_nights)
+    f, l = 0, n_obs_nights[0]
+    for i in range(n_nights):
+        yield i, f, l
+        if i < n_nights - 1:
+            f += n_obs_nights[i]
+            l += n_obs_nights[i+1]
+        
+        
 def weighted_brute_force(forward_model, templates_dict, iter_index, sregion, xcorr_options):
     """Performs a pseudo weighted cross-correlation via brute force RMS minimization and estimating the minumum with a quadratic.
 
@@ -95,8 +112,12 @@ def weighted_brute_force(forward_model, templates_dict, iter_index, sregion, xco
         tell_flux_hrc = forward_model.models_dict['lsf'].convolve_flux(tell_flux_hr, pars=pars)
         tell_flux_lr = np.interp(forward_model.models_dict['wavelength_solution'].build(pars), templates_dict['star'][:, 0], tell_flux_hrc, left=0, right=0)
         tell_weights = tell_flux_lr**2
+        bad = np.where(tell_flux_lr < 0.99)
+        tell_weights = np.ones_like(weights_init)
+        tell_weights[bad] = 0
         # Combine weights
         weights_init *= tell_weights
+        
     
     # Star weights depend on the information content.
     rvc, _ = compute_rv_content(templates_dict['star'][:, 0], templates_dict['star'][:, 1], snr=100, blaze=False, ron=0, width=pars[forward_model.models_dict['lsf'].par_names[0]].value)
@@ -114,7 +135,7 @@ def weighted_brute_force(forward_model, templates_dict, iter_index, sregion, xco
         wave_lr, model_lr = forward_model.build_full(pars, templates_dict)
         
         # Shift the stellar weights instead of recomputing the rv content.
-        star_weights_shifted = pcmath.doppler_shift(templates_dict['star'][:, 0], vels[i], flux=star_weights, interp='linear', wave_out=wave_lr)
+        star_weights_shifted = pcmath.doppler_shift(templates_dict['star'][:, 0], vels[i], flux=star_weights, interp='spline', wave_out=wave_lr)
         weights *= star_weights_shifted
         
         # Construct the RMS
@@ -126,8 +147,8 @@ def weighted_brute_force(forward_model, templates_dict, iter_index, sregion, xco
     xcorr_rv_init = vels[M] + forward_model.data.bc_vel
 
     # Fit with a polynomial
-    # Include 3 points on each side of min vel
-    use = np.arange(M-3, M+3).astype(int)
+    # Include 5 points on each side of min vel (11 total points)
+    use = np.arange(M-5, M+6).astype(int)
 
     try:
         pfit = np.polyfit(vels_for_rv[use], rmss[use], 2)
@@ -151,7 +172,7 @@ def weighted_brute_force(forward_model, templates_dict, iter_index, sregion, xco
 
     return ccf_result
 
-# Super silly and crude but moderately sensible.
+# Silly and crude but moderately sensible.
 def ccf_uncertainty(cc_vels, ccf, v0, n):
     
     # First normalize the RMS function
@@ -211,84 +232,7 @@ def crude_brute_force(forward_model, templates_dict, sregion):
     xcorr_star_vel = vels[np.nanargmin(rmss)]
     
     return xcorr_star_vel
-
-
-
-def modifed_stddev(rvs, weights, mus, n_obs_nights):
-    n_nights = mus.size
-    unc = np.full(n_nights, fill_value=np.nan)
-    n_orders, n_spec = rvs.shape
-    f, l = 0, n_obs_nights[0]
-    for i in range(n_nights):
-        rr, ww = rvs[:, f:l].flatten(), weights[:, f:l].flatten()
-        ng = np.where(ww > 0)[0].size
-        if ng in (0, 1):
-            unc[i] = np.nan
-        else:
-            unc[i] = pcmath.weighted_stddev_mumod(rr, ww, mus[i]) / np.sqrt(ng)
-            
-        if i < n_nights - 1:
-            f += n_obs_nights[i]
-            l += n_obs_nights[i+1]
     
- 
-# Wobble Method of combining RVs (Starting from single RVs)
-# pars[0:n] = rvs
-# pars[n:] = order offsets.
-@jit
-def rv_solver(pars, rvs, weights, n_obs_nights):
-    """Internal function to optimize the rv offsets between orders.
-    """
-    
-    n_ord = rvs.shape[0]
-    n_spec = rvs.shape[1]
-    n_nights = n_obs_nights.size
-    rvs_individual = pars[:n_spec]
-    order_offsets = pars[n_spec:]
-    term = np.empty(shape=(n_ord, n_spec), dtype=np.float64)
-    
-    for o in range(n_ord):
-        for i in range(n_spec):
-            term[o, i] = weights[o, i]**2 * (rvs_individual[i] - rvs[o, i] + order_offsets[o])**2
-            
-    rms = np.sqrt(np.nansum(term)) # Technically not an rms
-    
-    bad = np.where(term > 5*rms)
-    if bad[0].size > 0:
-        term[bad] = 0
-    rms = np.sqrt(np.nansum(term))
-    
-    return rms
-
-# Wobble Method of combining RVs (Starting from single RVs)
-# pars[0:n] = rvs
-# pars[n:] = order offsets.
-@jit
-def rv_solver_fast(pars, rvs, weights, n_obs_nights):
-    
-    n_ord, n_spec = rvs.shape
-    n_tot = n_ord * n_spec
-    n_nights = len(n_obs_nights)
-    rvs_nightly = pars[:n_nights]
-    order_offsets = pars[n_nights:]
-    diffs = np.zeros(shape=(n_ord, n_spec), dtype=np.float64)
-    
-    for o in range(n_ord):
-        f, l = 0, n_obs_nights[0]
-        for i in range(n_nights):
-            diffs[o, f:l] = weights[o, f:l]**2 * (rvs_nightly[i] - rvs[o, f:l] + order_offsets[o])**2
-            if i < n_nights - 1:
-                f += n_obs_nights[i]
-                l += n_obs_nights[i+1]
-    
-    good = np.where(weights > 0)
-    diffs  = diffs[good].flatten()
-    ss = np.argsort(diffs)
-    diffs[ss[-int(0.05 * n_tot):]] = 0
-    rms = np.sqrt(np.nansum(diffs)) # Technically not an rms
-    
-    return rms
- 
 # Computes the RV content per pixel.
 def compute_rv_content(wave, flux, snr=100, blaze=False, ron=0, R=None, width=None, sampling=None, wave_to_sample=None, lsf=None):
     """Computes the radial-velocity information content per pixel and for a whole swath.
@@ -371,7 +315,6 @@ def compute_rv_content(wave, flux, snr=100, blaze=False, ron=0, R=None, width=No
         rvc_tot = np.nansum(1 / rvc_per_pix[good]**2)**-0.5
         return rvc_per_pix, rvc_tot
         
-
 def compute_bisector_span(cc_vels, ccf, v0, n_bs=1000):
     """Computes the Bisector inverse slope of a given cross-correlation (RMS brute force) function.
 
@@ -437,7 +380,7 @@ def compute_bisector_span(cc_vels, ccf, v0, n_bs=1000):
     
     return line_bisectors, bis
 
-def detrend_rvs(rvs, vec, thresh=None):
+def detrend_rvs(rvs, vec, thresh=None, poly_order=1):
     
     if thresh is None:
         thresh = 0.5
@@ -446,7 +389,7 @@ def detrend_rvs(rvs, vec, thresh=None):
     if np.abs(pcc) < thresh:
         return rvs
     else:
-        pfit = np.polyfit(vec, rvs, 1)
+        pfit = np.polyfit(vec, rvs, poly_order)
         rvs_out = rvs - np.polyval(pfit, vec)
         return rvs_out
 
@@ -462,44 +405,72 @@ def compute_nightly_rvs_single_order(rvs, weights, n_obs_nights, flag_outliers=F
     n_spec = len(rvs)
     n_nights = len(n_obs_nights)
     
-    # Will hold the start and end index for each night.
-    f, l = 0, n_obs_nights[0]
+    # Initialize the nightly rvs and uncertainties
+    rvs_nightly = np.full(n_nights, np.nan)
+    unc_nightly = np.full(n_nights, np.nan)
+    
+    for i, f, l in nightly_iteration(n_obs_nights):
+        rr = rvs[f:l, :].flatten()
+        ww = weights[f:l, :].flatten()
+        rvs_nightly[i], unc_nightly[i] = pcmath.weighted_combine(rr, ww, yerr=None, err_type="empirical")
+            
+    return rvs_nightly, unc_nightly
+
+def compute_nightly_rvs_single_chunk(rvs, weights, n_obs_nights):
+    """Computes nightly RVs for a single order.
+
+    Args:
+        rvs (np.ndarray): The individual rvs array of shape (n_spec,).
+        weights (np.ndarray): The weights, also of shape (n_spec,).
+        n_obs_nights (np.ndarray): The array of length n_nights containing the number of observations on each night.
+    """
+    
+    # The number of spectra and nights
+    n_spec = len(rvs)
+    n_nights = len(n_obs_nights)
     
     # Initialize the nightly rvs and uncertainties
     rvs_nightly = np.full(n_nights, np.nan)
     unc_nightly = np.full(n_nights, np.nan)
     
-    for inight in range(n_nights):
-        r = rvs[f:l, :].flatten()
-        w = weights[f:l, :].flatten()
-        good = np.where(w > 0)[0]
-        ng = good.size
-        if ng == 0:
-            rvs_nightly[inight] = np.nan
-            unc_nightly[inight] = np.nan
-        elif ng == 1:
-            rvs_nightly[inight] = r[good[0]]
-            unc_nightly[inight] = np.nan
-        else:
-            if flag_outliers:
-                wavg = pcmath.weighted_mean(r, w)
-                wstddev = pcmath.weighted_stddev(r, w)
-                bad = np.where(np.abs(r - wavg) > thresh*wstddev)[0]
-                n_bad = bad.size
-                if n_bad > 0:
-                    w[bad] = 0
-            else:
-                n_bad = 0
-            rvs_nightly[inight] = pcmath.weighted_mean(r, w)
-            unc_nightly[inight] = pcmath.weighted_stddev(r, w) / np.sqrt(ng - n_bad)
-            
-        if inight < n_nights - 1:
-            f += n_obs_nights[inight]
-            l += n_obs_nights[inight + 1]
+    for i, f, l in nightly_iterable(n_obs_nights):
+        rr = rvs[f:l].flatten()
+        ww = weights[f:l].flatten()
+        rvs_nightly[i], unc_nightly[i] = pcmath.weighted_combine(rr, ww, yerr=None, err_type="empirical")
             
     return rvs_nightly, unc_nightly
-             
+   
+def bin_rvs_to_nights(jds, rvs, unc, err_type="empirical"):
+    """A separate function to bin RVs to 1 per night, primarily for use by external users for now.
+
+    Args:
+        jds (np.ndarray): The individual BJDs of length n_obs.
+        rvs (np.ndarray): The individual RVs of length n_obs.
+        unc (np.ndarray): The corresponding RV errors of length n_obs.
+        
+    Returns:
+        np.ndarray: The nightly BJDs.
+        np.ndarray: The nightly RVs.
+        np.ndarray: The nightly RV errors.
+    """
     
+    # Get nightly JDs
+    jds_nightly, n_obs_nights = get_nightly_jds(jds, sep=0.5)
+    
+    # Number of nights
+    n_nights = len(n_obs_nights)
+    
+    # Initialize nightly rvs and errors
+    rvs_nightly = np.zeros(n_nights, dtype=float)
+    unc_nightly = np.zeros(n_nights, dtype=float)
+    
+    # For each night, coadd RVs
+    for i, f, l in nightly_iteration(n_obs_nights):
+        rr, uncc = rvs[f:l].flatten(), unc[f:l].flatten()
+        ww = 1 / uncc**2
+        rvs_nightly[i], unc_nightly[i] = pcmath.weighted_combine(rr, ww, yerr=uncc, err_type="empirical")
+    return jds_nightly, rvs_nightly, unc_nightly
+             
 def compute_nightly_rvs_from_all(rvs, weights, n_obs_nights, flag_outliers=False, thresh=5):
     """Computes nightly RVs for a single order.
 
@@ -512,45 +483,16 @@ def compute_nightly_rvs_from_all(rvs, weights, n_obs_nights, flag_outliers=False
     n_orders, n_spec = rvs.shape
     n_nights = len(n_obs_nights)
     
-    # Will hold the start and end index for each night.
-    f, l = 0, n_obs_nights[0]
-    
     # Initialize the nightly rvs and uncertainties
-    rvs_nightly = np.zeros(n_nights)
-    unc_nightly = np.zeros(n_nights)
+    rvs_nightly = np.full(n_nights, np.nan)
+    unc_nightly = np.full(n_nights, np.nan)
     
-    for inight in range(n_nights):
-        r = rvs[:, f:l].flatten()
-        w = weights[:, f:l].flatten()
-        good = np.where(w > 0)[0]
-        ng = good.size
-        if ng == 0:
-            rvs_nightly[inight] = np.nan
-            unc_nightly[inight] = np.nan
-        elif ng == 1:
-            rvs_nightly[inight] = r[good[0]]
-            unc_nightly[inight] = np.nan
-        else:
-            if flag_outliers:
-                wavg = pcmath.weighted_mean(r, w)
-                wstddev = pcmath.weighted_stddev(r, w)
-                bad = np.where(np.abs(r - wavg) > thresh*wstddev)[0]
-                n_bad = bad.size
-                if n_bad > 0:
-                    w[bad] = 0
-            else:
-                n_bad = 0
-            rvs_nightly[inight] = pcmath.weighted_mean(r, w)
-            unc_nightly[inight] = pcmath.weighted_stddev(r, w) / np.sqrt(ng - n_bad)
-            
-        if inight < n_nights - 1:
-            f += n_obs_nights[inight]
-            l += n_obs_nights[inight + 1]
-            
+    for i, f, l in nightly_iterable(n_obs_nights):
+        rr = rvs[:, f:l].flatten()
+        ww = weights[:, f:l].flatten()
+        rvs_nightly[i], unc_nightly[i] = pcmath.weighted_combine(rr, ww, yerr=None, err_type="empirical")
     return rvs_nightly, unc_nightly
 
-
-
 def compute_relative_rvs_from_nights(rvs, rvs_nightly, unc_nightly, weights, n_obs_nights):
     """Combines RVs considering the differences between all the data points
 
@@ -591,13 +533,12 @@ def compute_relative_rvs_from_nights(rvs, rvs_nightly, unc_nightly, weights, n_o
         
     return np.copy(rvs[0, :]), np.zeros(n_spec) + 10, rvi, unci
 
-
 def combine_relative_rvs(rvs, weights, n_obs_nights):
-    """Combines RVs considering the differences between all the data points
-
+    """Combines RVs considering the differences between all the data points.
+    
     Args:
-        rvs (np.ndarray): RVs
-        weights (np.ndarray): Corresponding uncertainties
+        rvs (np.ndarray): RVs of shape n_orders, n_spec, n_chunks
+        weights (np.ndarray): Corresponding uncertainties of the same shape.
     """
     
     # Numbers
@@ -608,8 +549,8 @@ def combine_relative_rvs(rvs, weights, n_obs_nights):
     n_tot_chunks = n_orders * n_chunks
     
     # Determine differences and weights tensors
-    rvlij = np.zeros((n_tot_chunks, n_spec, n_spec))
-    wlij = np.zeros((n_tot_chunks, n_spec, n_spec))
+    rvlij = np.full((n_tot_chunks, n_spec, n_spec), np.nan)
+    wlij = np.full((n_tot_chunks, n_spec, n_spec), np.nan)
     for l in range(n_orders):
         for ichunk in range(n_chunks):
             i_quasi_chunk = l * n_chunks + ichunk
@@ -623,19 +564,14 @@ def combine_relative_rvs(rvs, weights, n_obs_nights):
     uncli = np.full(shape=(n_tot_chunks, n_spec), fill_value=np.nan)
     for l in range(n_tot_chunks):
         for i in range(n_spec):
-            good = np.where(wlij[l, i, :] > 0)[0]
-            ng = good.size
-            if ng == 0:
-                continue
-            elif ng == 1:
-                rvli[l, i] = rvlij[l, i, good[0]]
-                uncli[l, i] = np.nan
-            else:
-                rvli[l, i] = pcmath.weighted_mean(rvlij[l, i, :], wlij[l, i, :])
-                uncli[l, i] = pcmath.weighted_stddev(rvlij[l, i, :], wlij[l, i, :]) / np.sqrt(ng)
+            rr = rvlij[l, i, :]
+            ww = wlij[l, i, :]
+            rvli[l, i], uncli[l, i] = pcmath.weighted_combine(rr, ww, yerr=None, err_type="empirical")
     
+    # Weights
     wli = (1 / uncli**2) / np.nansum(1 / uncli**2, axis=0)
     
+    # Output arrays
     rvs_single_out = np.full(n_spec, fill_value=np.nan)
     unc_single_out = np.full(n_spec, fill_value=np.nan)
     rvs_nightly_out = np.full(n_nights, fill_value=np.nan)
@@ -645,39 +581,120 @@ def combine_relative_rvs(rvs, weights, n_obs_nights):
         wli[bad] = 0
         
     for i in range(n_spec):
-        good = np.where(wli[:, i] > 0)[0]
-        ng = good.size
-        if ng == 0:
-            continue
-        elif ng == 1:
-            rvs_single_out[i] = rvli[good[0], i]
-            unc_single_out[i] = uncli[good[0], i]
-        else:
-            rvs_single_out[i] = pcmath.weighted_mean(rvli[:, i], wli[:, i])
-            unc_single_out[i] = pcmath.weighted_stddev(rvli[:, i], wli[:, i]) / np.sqrt(ng)
+        rvs_single_out[i], unc_single_out[i] = pcmath.weighted_combine(rvli[:, i].flatten(), wli[:, i].flatten())
+        
+    for i, f, l in nightly_iteration(n_obs_nights):
+        rr = rvs_single_out[f:l]
+        uncc = unc_single_out[f:l]
+        ww = 1 / uncc**2
+        rvs_nightly_out[i], unc_nightly_out[i] = pcmath.weighted_combine(rr, ww, yerr=uncc)
+            
+    rvs_out = {"rvs": rvs_single_out, "unc": unc_single_out, "rvs_nightly": rvs_nightly_out, "unc_nightly" : unc_nightly_out}
+        
+    return rvs_out
+
+def combine_rvs_tfa(rvs, weights, n_obs_nights):
+    """Combines RVs considering the differences between all the data points
+
+    Args:
+        rvs (np.ndarray): RVs, of shape=(n_orders, n_spec, n_chunks).
+        weights (np.ndarray): Corresponding uncertainties with the same shape.
+    """
+    
+    # Numbers
+    n_orders, n_spec, n_chunks = rvs.shape
+    n_nights = len(n_obs_nights)
+    
+    # Rephrase problem as n_quasi_orders = n_tot_chunks = n_orders * n_chunks
+    n_tot_chunks = n_orders * n_chunks
+    
+    # Reshape variables
+    rvs_nonans = np.copy(rvs)
+    rvscp = np.copy(rvs)
+    rvs_nonans -= np.nanmedian(rvs_nonans)
+    bad = np.where(~np.isfinite(rvs_nonans))
+    rvs_nonans[bad] = 0
+    S = rvs_nonans.reshape((n_tot_chunks, n_spec))
+    rvscp = rvscp.reshape((n_tot_chunks, n_spec))
+    w = weights.reshape((n_tot_chunks, n_spec))
+    
+    # Compute helper variables
+    Aj = np.einsum("oj,oj->j", w**2, S)
+    Bj = np.einsum("oj->j", w**2)
+    Cm = np.einsum("mi,mi->m", w**2, S)
+    Dm = np.einsum("mi->m", w**2)
+    Hj = np.einsum("oj,o->j", w**2, Cm / Dm)
+    bad = np.where(~np.isfinite(Hj))[0]
+    Hj[bad] = 0
+    Koij = np.zeros((n_tot_chunks, n_spec, n_spec))
+    for o in range(n_tot_chunks):
+        for i in range(n_spec):
+            for j in range(n_spec):
+                Koij[o, i, j] = w[o, j]**2 * w[o, i]**2 / Dm[o]
+    
+    bad = np.where(~np.isfinite(Koij))[0]
+    Koij[bad] = 0
+    
+    Pij = np.einsum("oij->ij", Koij)
+    Qj = Aj / Bj
+    Rj = Hj / Bj
+    Tj = 1 / Bj
+    bad = np.where(~np.isfinite(Qj) | ~np.isfinite(Rj) | ~np.isfinite(Tj))[0]
+    Qj[bad] = 0
+    Rj[bad] = 0
+    Tj[bad] = 0
+    Pij_twiddle = np.zeros((n_spec, n_spec))
+    for i in range(n_spec):
+        Pij_twiddle[i, :] = Tj * Pij[i, :]
+    
+    # The individual RVs
+    V = np.dot((Qj - Rj), np.linalg.inv(np.eye(n_spec) - Pij_twiddle.T))
+    
+    # Compute helper variables for the offsets
+    Lm = np.einsum("mi,i->m", w**2, Aj / Bj)
+    bad = np.where(~np.isfinite(Lm))[0]
+    Lm[bad] = 0
+    Zmoi = np.zeros((n_tot_chunks, n_tot_chunks, n_spec))
+    for m in range(n_tot_chunks):
+        for o in range(n_tot_chunks):
+            for i in range(n_spec):
+                Zmoi[m, o, i] = w[m, i]**2 * w[o, i]**2 / Bj[i]
+                
+    bad = np.where(~np.isfinite(Zmoi))[0]
+    Zmoi[bad] = 0
+    
+    Gmo = np.einsum("moi->mo", Zmoi)
+    deltam = Cm / Dm
+    pim = Lm / Dm
+    thetam = 1 / Dm
+    bad = np.where(~np.isfinite(deltam) | ~np.isfinite(pim) | ~np.isfinite(thetam))[0]
+    deltam[bad] = 0
+    pim[bad] = 0
+    thetam[bad] = 0
+    Gmo_twiddle = np.zeros((n_tot_chunks, n_tot_chunks))
+    for m in range(n_tot_chunks):
+        Gmo_twiddle[m, :] = thetam * Gmo[m, :]
+        
+    gamma = np.dot((deltam - pim), np.linalg.inv(np.eye(n_tot_chunks) - Gmo_twiddle))
+    
+    # With gamma, construct the offset RVs
+    rvs_offset = np.zeros((n_tot_chunks, n_spec))
+    for o in range(n_tot_chunks):
+        rvs_offset[o, :] = rvscp[o, :] - gamma[o]
+        
+    # With offset RVs, actually perform the offsets and compute weighted means & corresponding errors
+    rvs_single_out = np.zeros(n_spec)
+    unc_single_out = np.zeros(n_spec)
+    rvs_nightly_out = np.zeros(n_nights)
+    unc_nightly_out = np.zeros(n_nights)
+    
+    # For each observation, compute the coadded RVs
+    for i in range(n_spec):
+        rvs_single_out[i], unc_single_out[i] = pcmath.weighted_combine(rvs_offset[:, i], weights[:, i])
         
     f, l = 0, n_obs_nights[0]
-
     for i in range(n_nights):
-        if n_obs_nights[i] == 1 and np.isfinite(rvs_single_out[f]):
-            rvs_nightly_out[i] = rvs_single_out[f]
-            unc_nightly_out[i] = unc_single_out[f]
-        else:
-            ww = 1 / unc_single_out[f:l]**2
-            rr = rvs_single_out[f:l]
-            unc_all = unc_single_out[f:l]
-            good = np.where(ww > 0)[0]
-            ng = good.size
-            if ng == 0:
-                rvs_nightly_out[i] = np.nan
-                unc_nightly_out[i] = np.nan
-            elif ng == 1:
-                rvs_nightly_out[i] = rr[good[0]]
-                unc_nightly_out[i] = unc_all[good[0]]
-            else:
-                rvs_nightly_out[i] = pcmath.weighted_mean(rr[good], ww[good])
-                unc_nightly_out[i] = pcmath.weighted_stddev(rr[good], ww[good]) / np.sqrt(ng)
-            
+        rvs_nightly_out[i], unc_nightly_out[i] = pcmath.weighted_combine(rvs_offset[:, f:l].flatten(), weights[:, f:l].flatten())
         if i < n_nights - 1:
             f += n_obs_nights[i]
             l += n_obs_nights[i+1]
